/*
 * libuni.h
 * 
 * Started 08/04/2003
 * Matt Gray <matt@aps.umn.edu>
 *
 * Purpose: Define some data types and serve as the header for libuni.
 */

 #include <stdio.h>

/*
 * struct unifile
 * 
 * Basic structure for passing unifspec data between functions.
 */

struct unifile
{
	/* The X1 variable. The header line containing this data will be
	 * automatically written from this data.
	 */
	double x1;

	/* The XN variable. This is the largest X (xmax) */
	double xn; 

	double ymin, ymax;

	/* DX. Automatically included in header at writing time (see above) */
	double dx;

	/* Single-dimension array containing num_elems double values */
	double *data;

	/* Number of elements in the data array above */
	long int num_elems;

	int header_lines;

	char **header;
};

/* uni_write_file - take data in in through a struct unifile, and write out
 * to a given filename. This will be a high-level procedure, calling smaller
 * procedures like create_header to allow for flexibility. After all, the user
 * should be able to make their own header files, and not have to rely on the
 * code herein.
 *
 * uni_write_file_old is identical, only it calls uni_create_header_old to 
 * create a unifspec file using the old WVL1 format..
 *
 * Parameters
 *  by value: char *filename
 *  by ref  : struct unifile ufile
 * 
 * Returns
 *  0 on success
 *  nonzero error code on failure (see code)
 */
int
uni_write_file (char *filename, struct unifile *ufile);

/* uni_read_file - open filename, and generate a struct unifile, passed by
 * reference. This function will assume that it has been passed an empty
 * pointer and will allocate memory appropriately. Error codes should be 
 * checked.
 *
 * Parameters
 *  by value: char *filename
 *  by ref  : struct unifile ufile
 * 
 * Returns
 *  0 on success
 *  nonzero error code on failure (see code)
 */
int
uni_read_file (char *filename, struct unifile *ufile);


/* uni_create_header - uni_create_header will convert the array of strings
 * user_header into a suitable block of characters for writing to a file.
 * It is assumed that the required elements in the header (e.g. the X1,DX
 * or WVL1,DWVL line is included in said header. The proper way
 * to determine if such a line needs to be added is with uni_readkey
 * and to write one, use uni_writekey.
 *
 * Parameters
 *  by value: char **user_header, int header_lines
 *  by ref  : char *header, int bytes
 * 
 * Returns
 *  0 on success
 *  nonzero error code on failure (see code)
 */
int
uni_create_header (char **header, long int *bytes, struct unifile *ufile);

/* uni_new_header - Initialize the header constructs, adding the terminating
 * '#^^M' on a line all by itself, as well as culling the standard information
 * from the unifile struct.
 */ 
int
uni_new_header (struct unifile *ufile);

/* uni_readkey - Read a key from the header construct, where the key is in the
 * format:
 * #KEYNAME=  DATA\0
 */

int
uni_readkey (char **var, const char *keyname, struct unifile *ufile);

/* uni_writekey - Write a key into the header construct, checking for its
 * existence (by trying uni_readkey). If it exists, uni_deletekey will be 
 * called.
 */

int
uni_writekey (char *var, const char *keyname, struct unifile *ufile);

/* uni_deletekey - Removes a key if it exists from the header. */

int
uni_deletekey (const char *keyname, struct unifile *ufile);

/* uni_iskey - checks if such a key as keyname exists. If so, returns true. */

int
uni_iskey (const char *keyname, struct unifile *ufile);

/* uni_findkey - finds and returns the index of the key keyname */

int
uni_findkey (const char *keyname, struct unifile *ufile);

/* uni_parse_header
 * Previously used to perform all sorts of stuff, now modified to simply
 * scan a raw header (in a char array) and parse it into a unifile
 * structure. This is done on a very basic level, ignoring the abstractions
 * readkey, etc. Basically, all CR/LFs are converted to null characters, and 
 * placed in sequential order into the struct. Also, X1 and DX are placed in
 * the struct, after being read from the raw header. This information is not
 * replicated in the array of strings in the struct, due to the fact that
 * inconsistencies could be introduced where the variables dx and x1 differ
 * from the actual X1,DX= string. Because this string is central to unifspec,
 * it is crucial that this information is isolated from possible confusion.
 * 
 * Parameters 
 *  by value: char *header, int bytes
 *  by ref  : struct unifile *ufile
 * 
 * Returns
 *  0 on success
 *  nonzero error code on failure (see code)
 */
int
uni_parse_header (char *header, int bytes, struct unifile *ufile);

/* uni_find_data - Given an open file pointer fp, uni_find_data will scan
 * through the given file from the beginning, looking for the special line 
 * that ends in '#^^M' ('#^' followed by a CR-LF). The function will then set
 * the integer given to the location of the first _byte_ of data.
 *
 * Parameters
 *  by value: FILE *fp
 *  by ref  : int pos
 * 
 * Returns
 *  0 on success
 *  nonzero error code on failure (see code)
 */ 
int
uni_find_data (FILE *fp, int *pos);

/* uni_read_data - Given an open file pointer fp, uni_read_data will allocate
 * and scan the data into an array of type double, and set num_elems to the
 * number of elements in the array.
 *
 * Parameters
 *  by value: FILE *fp
 *  by ref  : struct unifile ufile
 * 
 * Returns
 *  0 on success
 *  nonzero error code on failure (see code)
 */
int
uni_read_data (FILE *fp, double **data, long int *num_elems);

/* uni_read_header - Given an open file pointer fp, uni_read_header will
 * allocate for and scan the header of the file, storing the unparsed string
 * in char *header, and setting bytes to the number of bytes in the header.
 *
 * Parameters
 *  by value: FILE *fp
 *  by ref  : char *header, long int bytes
 *
 * Returns
 *  0 on success
 *  nonzero error code on failure (see code)
 */
int
uni_read_header (FILE *fp, char **header, long int *bytes);

/* uni_write_data - Given an open file pointer fp, uni_write_data will
 * write num_elems of data from the array data to the open file. There is
 * one caveat, however - the file position must be in the proper place. This
 * is true if you have just ran uni_write_header, but if you have performed
 * any other operations like fseek(), uni_write_data is oblivious. If using
 * uni_write_data after some number of fileops, use uni_find_data() - assuming
 * of course, that you have written a header!
 *
 * Parameters
 *  by value: FILE *fp
 *  by ref  : struct unifile *ufile
 *
 * Returns
 *  0 on success
 *  nonzero error code on failure (see code)
 */
int
uni_write_data (FILE *fp, double *data, long int num_elems);

/* uni_write_header - Given an open file pointer fp, uni_write_header will
 * write bytes of header to the file. Because this is the header,
 * uni_write_header will automatically seek to the beginning of the file.
 * This is meant to be used before calling uni_write_data().
 *
 * Parameters
 *  by value: FILE *fp, char *header, long int bytes
 *
 * Returns
 *  0 on success
 *  nonzero error code on failure (see code)
 */
int
uni_write_header (FILE *fp, char *header, long int bytes);

/* uni_create - Create a unifspec struct out of two arrays */
int
uni_create (double *xdata, double *ydata, long int xelems, long int yelems, struct unifile *ufile);

/* uni_free - Given a struct with dynamically defined elements that are not
 * NULL pointers, uni_free will attempt to call free on all the dynamic parts
 * of the struct, in order to clean it for re-use (e.g. opening a new file).
 *
 * Parameters
 *  by value: none
 *  by reference: struct unifile ufile
 *
 * Returns
 *  0 on success
 *  nonzero error code on failure (see code)
 */
int
uni_free (struct unifile *ufile);

/* uni_calc_meta - Given a struct, u_c_m will scan the data array, and
 * figure out xmax, ymin, and ymax, and store them in the structure.
 * This functon is normally automatically called when reading in a new
 * unifile through the main wrapper (e.g. uni_read_file) but should be
 * called after modifying the data contained in the data element, in
 * order to keep the max,min consistent.
 *
 * Paramenters
 *  by value: none
 *  by reference: struct unifile ufile
 *
 * Returns
 *  0 on success
 *  nonzero error code on failure (see code)
 */
int
uni_calc_meta (struct unifile *ufile);

/* uni_normalize_to - Given a reference unifile and a wavelength range,
 * figure out some scaling factor to normalize ufile to ref_ufile using
 * the specified range as a guide.
 *
 * Parameters
 *
 * Returns
 */
int
uni_normalize_to (struct unifile *ref_ufile, struct unifile *ufile, double minwvl, double maxwvl);

int
qfit (double *x, double *f, double *coef);
