/*
 * uni_write_file.c
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "libuni.h"

/* uni_write_file
 * Provides an all-in-one wrapper to fully write a consistent unifile struct 
 * out to disk. Handles everything: the header, the data, etc. Will not over-
 * write an existing file.
 */
int
uni_write_file (char *filename, struct unifile *ufile)
{
	int result;
	FILE *fp;
	char *header;
	long int header_bytes;

	/* If we are dealing with a '-', set fp to stdout */

	if (strlen (filename) > 0 && filename[0] == '-')
		{
			fp = stdout;
		}
	else
		{

			/* Try to open the file */
			fp = fopen (filename, "rb");

			if (fp == NULL)
				{
					/* Good, the file doesn't exist */
					fp = fopen (filename, "wb");
					if (fp == NULL)
						return 1;
				}
			else
				{
					/* File exists */
					#ifdef _DEBUG
					fprintf (stderr, "ERROR: uni_write_file.c - file %s exists!\n", filename);
					#endif
					return 1;
				}
		}
	
	/* uni_create_header handles adding the required and optional keys,
	 * generated from the struct - it also checks to make sure that 
	 * the end of header sentry is present, or if the header is even valid. If
	 * the header does not exist, it will generate an entirely new one.
	 */
	result = uni_create_header (&header, &header_bytes, ufile);

	if (result)
		{
			/* Problem with uni_create_header */
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_write_file.c - uni_create_header returned %d\n", result);
			#endif
			return 2;
		}

	/* Write it out to the file. */
	result = uni_write_header(fp, header, header_bytes);

	if (result)
		{
			/* Problem with uni_write_header */
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_write_file.c - uni_write_header returned %d\n", result);
			#endif
			return 3;
		}
	
	/* Write out the data, now that the header is in place. */
	result = uni_write_data(fp, ufile[0].data, ufile[0].num_elems);

	if (result)
		{
			/* Problem with uni_write_data */
			#ifdef _DEBUG
			fprintf (stderr, "ERROR: uni_write_file.c - uni_write_data returned %d\n", result);
			#endif
		}

	if (fp != stdout)
		fclose (fp);
	
	return 0;
}
